/*
 *  Sample EMS plugin: PersonalityPlugin
 *
 *  Filename: PersonalityPluginTran.cpp
 *
 *  Last edited: Wednesday, December 18, 1996
 *
 *  Author: Scott Manjourides
 *
 *  Copyright 1995, 1996 QUALCOMM Inc.
 *
 *  Send comments and questions to <emsapi-info@qualcomm.com>
 */

#include "stdafx.h"
#ifdef _DEBUG
#include <ctype.h>
#endif // _DEBUG

#include <afxwin.h> // MFC & Windows master header
#include "resource.h" // Resource IDs for dialogs and icons
#include "PersonalityPlugin.h" // CPersonalityPluginDLL

#include <malloc.h> // malloc, free
#include <string.h> // strlen, strdup, strrev
#include <stdlib.h>
#include <time.h>

#include <windows.h>
#include "ems-win.h" // The EMS API

#include "mimetype.h"

/*****************************************************************************/
/* CONSTANTS */

static const int kPluginID = 197506022;
static const char *kPluginDescription = "PersonalityPlugin Plugin by Michael John Radwin, v1.0";
static const unsigned long kPluginIconID = IDI_MAIN;

static const long kJiveID = 1L;
static const long kB1FFID = 2L;
static const long kChefID = 3L;
static const int kNumTrans    = 3;

static const char *kFileErrorStr = "File I/O Error";
static const char *kOutOfMem = "Out of Memory";

/*****************************************************************************/
/* GLOBALS */

// This externs into PersonalityPlugin.cpp
extern CPersonalityPluginDLL theDLL;

static struct TransInfoStruct {
	char *description;
	long type;
	long subtype;
	unsigned long flags;
	char *MimeType;
	char *MimeSubtype;
	unsigned long nIconID;
} gTransInfo[] = {
	{
		"Jive",
		EMST_LANGUAGE,
		kJiveID,
		EMSF_ON_REQUEST|EMSF_TOOLBAR_PRESENCE,
		"text", "plain",
		IDI_JIVE
	},
	{
		"B1FF",
		EMST_LANGUAGE,
		kB1FFID,
		EMSF_ON_REQUEST|EMSF_TOOLBAR_PRESENCE,
		"text", "plain",
		IDI_B1FF
	},
	{
		"Swedish Chef",
		EMST_LANGUAGE,
		kChefID,
		EMSF_ON_REQUEST|EMSF_TOOLBAR_PRESENCE,
		"text", "plain",
		IDI_CHEF
	}
};

/*****************************************************************************/
/* MACROS */

#define safefree(p) { if (p) { free(p); p=NULL; } }

/*****************************************************************************/
/* LOCAL FUNCTION PROTOTYPES */

// Generalized functions
int CheckValidContext(const long trans_id, const long context);
int CheckValidMimeType(const long trans_id, const emsMIMEtypeP in_mime);
void DoIconInit(const long trans_id, HICON FAR*FAR* trans_icon);

/*****************************************************************************/
/* TRANSLATER API FUNCTIONS */

/*****\
*
* ems_plugin_version: Get the version of the API used for this plugin
*
\*****/

extern "C" long WINAPI ems_plugin_version(
    short FAR* api_version      /* Place to return api version */
)
{
	*api_version = EMS_VERSION;
	return (EMSR_OK);
}

/* - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - */

/*****\
*
* ems_plugin_init: Initialize plugin and get its basic info
*
\*****/

extern "C" long WINAPI ems_plugin_init(
    void FAR*FAR* globals,          /* Out: Return for allocated instance structure */
    short eudAPIVersion,        /* In: The API version eudora is using */
    emsMailConfigP mailConfig,  /* In: Eudoras mail configuration */
    emsPluginInfoP pluginInfo   /* Out: Return Plugin Information */
)
{
	// We don't use 'globals'

	if (pluginInfo)
	{
		pluginInfo->numTrans = (long) kNumTrans;
		pluginInfo->desc     = (LPSTR) strdup(kPluginDescription);
		pluginInfo->id       = (long) kPluginID;

		DoIconInit(-1, &(pluginInfo->icon));
	}

	return (EMSR_OK);
}

/* - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - */

/*****\
*
* ems_translator_info: Get details about a translator in a plugin
*
\*****/

extern "C" long WINAPI ems_translator_info(
    void FAR* globals,          /* Out: Return for allocated instance structure */
    emsTranslatorP transInfo    /* In/Out: Return Translator Information */
)
{
	if ((transInfo) && (gTransInfo))
	{
		const long id = (long) transInfo->id;

		if ((id <= 0) || (id > kNumTrans))
			return (EMSR_INVALID_TRANS);

		const TransInfoStruct *InfoPtr = gTransInfo + (id - 1);

		transInfo->type = (long) InfoPtr->type;
		transInfo->flags = (ULONG) InfoPtr->flags;
		transInfo->desc = (LPSTR) strdup(InfoPtr->description);

		DoIconInit(id, &(transInfo->icon));
	}

	return (EMSR_OK);
}

/* - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - */

/*****\
*
* ems_can_translate_file:
*     Check and see if a translation can be performed (file version)
*
\*****/

extern "C" long WINAPI ems_can_translate(
    void FAR* globals,            /* Out: Return for allocated instance structure */
    emsTranslatorP transInfo,     /* In: Translator Info */
    emsDataFileP inTransData,     /* In: What to translate */
    emsResultStatusP transStatus  /* Out: Translations Status information */
)
{
	if ((transInfo) && (inTransData))
	{
		const long id = (long) transInfo->id;

		if ((id <= 0) || (id > kNumTrans))
			return (EMSR_INVALID_TRANS);

		const long context = inTransData->context;
		const emsMIMEtypeP in_mime = inTransData->info;

		if ( (CheckValidContext(id, context)) && (CheckValidMimeType(id, in_mime)) )
		{
			return (EMSR_NOW);
		}
	}

	return (EMSR_CANT_TRANS);
}

/* - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - */

/*****\
*
* ems_translate_file: Actually perform a translation on a file
*
\*****/

extern FILE *jivein, *jiveout;
extern int jivelex();

extern FILE *b1ffin, *b1ffout;
extern int b1fflex();

extern FILE *chefin, *chefout;
extern int cheflex();

extern "C" long WINAPI ems_translate_file(
    void FAR* globals,          /* Out: Return for allocated instance structure */
    emsTranslatorP transInfo,       /* In: Translator Info */
    emsDataFileP inFileData,        /* In: What to translate */
    emsProgress progress,       /* Func to report progress/check for abort */
    emsDataFileP outFileData,       /* Out: Result of the translation */
    emsResultStatusP transStatus /* Out: Translations Status information */
)
{
	const char *in_file = inFileData->fileName;
	const char *out_file = outFileData->fileName;

	FILE *inFile = fopen(in_file, "rb");
	FILE *outFile = fopen(out_file, "wb");

	if (inFile == NULL)
	{
		if (transStatus)
		{
			transStatus->error = (LPSTR) strdup(kFileErrorStr);
			transStatus->code = (long) __LINE__;
		}

		return (EMSR_NO_INPUT_FILE);
	}

	if (!outFile)
		return (EMSR_UNKNOWN_FAIL);

	switch(transInfo->id) {
	case kJiveID: 
	    jivein = inFile;
	    jiveout = outFile;
	    jivelex();
	    break;
	case kB1FFID:
	    srand((unsigned) time(NULL));
	    b1ffin = inFile;
	    b1ffout = outFile;
	    b1fflex();
	    break;
	case kChefID:
	    chefin = inFile;
	    chefout = outFile;
	    cheflex();
	    break;
	default:
	    fprintf(outFile, "BAD FILTER!\n");
	}

	fclose(inFile);
	fclose(outFile);

	// Required to set the returned MIME type
	if (outFileData)
		outFileData->info = make_mime_type("text", "plain", "1.0");

	return (EMSR_OK);
}

/* - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - */

/*****\
*
* ems_plugin_finish: End use of a plugin and clean up
*
\*****/

extern "C" long WINAPI ems_plugin_finish(
    void FAR* globals           /* Out: Return for allocated instance structure */
)
{ 
	// We don't use 'globals'
	return (EMSR_OK); 
}

/* - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - */

/*****\
*
* ems_free: Free memory allocated by EMS plug-in
*
\*****/

extern "C" long WINAPI ems_free(
    void FAR* mem               /* Memory to free */
)
{
	if (mem)
		safefree(mem);

	return (EMSR_OK);
}

/*****************************************************************************/
/* HELPER FUNCTIONS */

static int CheckValidContext(
	const long trans_id,  /* ID of translator */
	const long context     /* Context for check; e.g. EMSF_ON_xxx */
)
{
	return ((context & gTransInfo[trans_id-1].flags) ? 1 : 0);
}

/* - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - */

static int CheckValidMimeType(
    const long trans_id,             /* ID of translator to check */
    const emsMIMEtypeP in_mime        /* MIME type of incomming data */
)
{
	char *pType = gTransInfo[trans_id-1].MimeType;
	char *pSubtype = gTransInfo[trans_id-1].MimeSubtype;

	if ( ((!pType) || (strcmp(in_mime->type, pType) == 0))
		&& ((!pSubtype) || (strcmp(in_mime->subType, pSubtype) == 0)) )
		return TRUE;

	return FALSE;
}

/* - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - */

static void DoIconInit(
    const long trans_id,         /* ID of translator to get info for */
    HICON FAR*FAR* icon   /* Return for icon data */
)
{
	AFX_MANAGE_STATE(AfxGetStaticModuleState());

	if (!icon)
		return;

	*icon = (HICON *)malloc(sizeof(HICON));

	if (trans_id < 0) /* Main plugin icon, not specific translator */
	{
		**icon = theDLL.LoadIcon(kPluginIconID); /* 32x32 */
	}
	else /* The actual translators */
	{
		**icon = theDLL.LoadIcon(gTransInfo[trans_id-1].nIconID); /* 16x16 */
	}
}
